@@ -11,7 +11,7 @@
  * Copyright (C) 2005 - 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2005 Dan Winship <danw@src.gnome.org>
  * Copyright (C) 2005 Stanislav Brabec <sbrabec@suse.de>
- * Copyright (C) 2005 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2006 - 2011 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2007 Leonardo Boshell <lb@kmc.com.co>
  * Copyright (C) 2008 Phil Longstaff <plongstaff@rogers.com>
@@ -49,6 +49,8 @@
 #include <libgda/gda-row.h>
 #include <libgda/gda-enums.h>
 #include <libgda/gda-data-handler.h>
+#include <libgda/gda-server-provider.h>
+#include <libgda/gda-server-provider-extra.h>
 #include <string.h>
 #ifdef HAVE_LOCALE_H
 #ifndef G_OS_WIN32
@@ -60,15 +62,9 @@
 #include "csv.h"
 
 extern gchar *gda_lang_locale;
-#if GLIB_CHECK_VERSION(2,31,7)
 static GRecMutex init_rmutex;
 #define MUTEX_LOCK() g_rec_mutex_lock(&init_rmutex)
 #define MUTEX_UNLOCK() g_rec_mutex_unlock(&init_rmutex)
-#else
-static GStaticRecMutex init_mutex = G_STATIC_REC_MUTEX_INIT;
-#define MUTEX_LOCK() g_static_rec_mutex_lock(&init_mutex)
-#define MUTEX_UNLOCK() g_static_rec_mutex_unlock(&init_mutex)
-#endif
 static void gda_data_model_class_init (gpointer g_class);
 
 static xmlNodePtr gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols, 
@@ -835,7 +831,7 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
  * gda_data_model_set_values:
  * @model: a #GdaDataModel object.
  * @row: row number.
- * @values: (element-type GLib.Value) (transfer none) (allow-none): a list of #GValue (or %NULL), one for at most the number of columns of @model
+ * @values: (element-type GObject.Value) (transfer none) (allow-none): a list of #GValue (or %NULL), one for at most the number of columns of @model
  * @error: a place to store errors, or %NULL
  *
  * In a similar way to gda_data_model_set_value_at(), this method modifies a data model's contents
@@ -1023,7 +1019,7 @@ gda_data_model_iter_set_value (GdaDataModel *model, GdaDataModelIter *iter, gint
 /**
  * gda_data_model_append_values:
  * @model: a #GdaDataModel object.
- * @values: (element-type GLib.Value) (allow-none): #GList of #GValue* representing the row to add.  The
+ * @values: (element-type GObject.Value) (allow-none): #GList of #GValue* representing the row to add.  The
  *          length must match model's column count.  These #GValue
  *	    are value-copied (the user is still responsible for freeing them).
  * @error: a place to store errors, or %NULL
@@ -1129,7 +1125,7 @@ gda_data_model_remove_row (GdaDataModel *model, gint row, GError **error)
 /**
  * gda_data_model_get_row_from_values:
  * @model: a #GdaDataModel object.
- * @values: (element-type GLib.Value): a list of #GValue values (no %NULL is allowed)
+ * @values: (element-type GObject.Value): a list of #GValue values (no %NULL is allowed)
  * @cols_index: (array): an array of #gint containing the column number to match each value of @values
  *
  * Returns the first row where all the values in @values at the columns identified at
@@ -1229,7 +1225,7 @@ gda_data_model_send_hint (GdaDataModel *model, GdaDataModelHint hint, const GVal
  * data models where some rows may be missing because the LDAP search has reached a limit
  * imposed by the LDAP server.
  *
- * Returns: (transfer none) (element-type GError) (array zero-terminated=1): a pointer to a %NULL terminated array of #GError, or %NULL.
+ * Returns: (transfer none) (element-type GLib.Error) (array zero-terminated=1): a pointer to a %NULL terminated array of #GError, or %NULL.
  *
  * Since: 4.2.6
  *
@@ -1292,18 +1288,15 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 		gint size;
 		xmlDocPtr xml_doc;
 
-		if (options) {
-			GdaHolder *holder;
-
-			holder = gda_set_get_holder (options, "NAME");
-			if (holder) {
-				const GValue *value;
-				value = gda_holder_get_value (holder);
-				if (value && (G_VALUE_TYPE (value) == G_TYPE_STRING))
-					name = g_value_get_string ((GValue *) value);
-				else
-					g_warning (_("The '%s' parameter must hold a string value, ignored."), "NAME");
-			}
+		GdaHolder *holder;
+		holder = options ? gda_set_get_holder (options, "NAME") : NULL;
+		if (holder) {
+			const GValue *value;
+			value = gda_holder_get_value (holder);
+			if (value && (G_VALUE_TYPE (value) == G_TYPE_STRING))
+				name = g_value_get_string ((GValue *) value);
+			else
+				g_warning (_("The '%s' parameter must hold a string value, ignored."), "NAME");
 		}
 		
 		xml_node = gda_data_model_to_xml_node (model, cols, nb_cols, rows, nb_rows, name);
@@ -1328,109 +1321,107 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 		gboolean invalid_as_null = FALSE;
 
 		retstring = g_string_new ("");
-		if (options) {
-			GdaHolder *holder;
+		GdaHolder *holder;
 
-			holder = gda_set_get_holder (options, "SEPARATOR");
-			if (holder) {
-				const GValue *value;
-				value = gda_holder_get_value (holder);
-				if (value && (G_VALUE_TYPE (value) == G_TYPE_STRING)) {
-					const gchar *str;
+		holder = options ? gda_set_get_holder (options, "SEPARATOR") : NULL;
+		if (holder) {
+			const GValue *value;
+			value = gda_holder_get_value (holder);
+			if (value && (G_VALUE_TYPE (value) == G_TYPE_STRING)) {
+				const gchar *str;
 
-					str = g_value_get_string ((GValue *) value);
-					if (str && *str)
-						sep = *str;
-				}
-				else
-					g_warning (_("The '%s' parameter must hold a string value, ignored."), "SEPARATOR");
+				str = g_value_get_string ((GValue *) value);
+				if (str && *str)
+					sep = *str;
 			}
-			holder = gda_set_get_holder (options, "QUOTE");
-			if (holder) {
-				const GValue *value;
-				value = gda_holder_get_value (holder);
-				if (value && (G_VALUE_TYPE (value) == G_TYPE_STRING)) {
-					const gchar *str;
+			else
+				g_warning (_("The '%s' parameter must hold a string value, ignored."), "SEPARATOR");
+		}
+		holder = options ? gda_set_get_holder (options, "QUOTE") : NULL;
+		if (holder) {
+			const GValue *value;
+			value = gda_holder_get_value (holder);
+			if (value && (G_VALUE_TYPE (value) == G_TYPE_STRING)) {
+				const gchar *str;
 
-					str = g_value_get_string ((GValue *) value);
-					if (str && *str)
-						quote = *str;
-				}
-				else 
-					g_warning (_("The '%s' parameter must hold a string value, ignored."), "QUOTE");
-			}
-			holder = gda_set_get_holder (options, "FIELD_QUOTE");
-			if (holder) {
-				const GValue *value;
-				value = gda_holder_get_value (holder);
-				if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
-					field_quote = g_value_get_boolean ((GValue *) value);
-				else 
-					g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "FIELD_QUOTE");
+				str = g_value_get_string ((GValue *) value);
+				if (str && *str)
+					quote = *str;
 			}
+			else
+				g_warning (_("The '%s' parameter must hold a string value, ignored."), "QUOTE");
+		}
+		holder = options ? gda_set_get_holder (options, "FIELD_QUOTE") : NULL;
+		if (holder) {
+			const GValue *value;
+			value = gda_holder_get_value (holder);
+			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
+				field_quote = g_value_get_boolean ((GValue *) value);
+			else
+				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "FIELD_QUOTE");
+		}
 
-			holder = gda_set_get_holder (options, "NULL_AS_EMPTY");
-			if (holder) {
-				const GValue *value;
-				value = gda_holder_get_value (holder);
-				if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
-					null_as_empty = g_value_get_boolean ((GValue *) value);
-				else 
-					g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "NULL_AS_EMPTY");
-			}
+		holder = options ? gda_set_get_holder (options, "NULL_AS_EMPTY") : NULL;
+		if (holder) {
+			const GValue *value;
+			value = gda_holder_get_value (holder);
+			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
+				null_as_empty = g_value_get_boolean ((GValue *) value);
+			else
+				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "NULL_AS_EMPTY");
+		}
 
-			holder = gda_set_get_holder (options, "INVALID_AS_NULL");
-			if (holder) {
-				const GValue *value;
-				value = gda_holder_get_value (holder);
-				if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
-					invalid_as_null = g_value_get_boolean ((GValue *) value);
-				else 
-					g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "INVALID_AS_NULL");
-			}
+		holder = options ? gda_set_get_holder (options, "INVALID_AS_NULL") : NULL;
+		if (holder) {
+			const GValue *value;
+			value = gda_holder_get_value (holder);
+			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
+				invalid_as_null = g_value_get_boolean ((GValue *) value);
+			else
+				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "INVALID_AS_NULL");
+		}
 
-			holder = gda_set_get_holder (options, "NAMES_ON_FIRST_LINE");
-			if (!holder)
-				holder = gda_set_get_holder (options, "FIELDS_NAME");
-			if (holder) {
-				const GValue *value;
-				value = gda_holder_get_value (holder);
-				if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN)) {
-					if (g_value_get_boolean (value)) {
-						gint col;
-						gint *rcols;
-						gint rnb_cols;
+		holder = options ? gda_set_get_holder (options, "NAMES_ON_FIRST_LINE") : NULL;
+		if (!holder && options)
+			holder = gda_set_get_holder (options, "FIELDS_NAME");
+		if (holder) {
+			const GValue *value;
+			value = gda_holder_get_value (holder);
+			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN)) {
+				if (g_value_get_boolean (value)) {
+					gint col;
+					gint *rcols;
+					gint rnb_cols;
 						
-						if (cols) {
-							rcols = (gint *)cols;
-							rnb_cols = nb_cols;
-						}
-						else {
-							gint i;
-							
-							rnb_cols = gda_data_model_get_n_columns (model);
-							rcols = g_new (gint, rnb_cols);
-							for (i = 0; i < rnb_cols; i++)
-								rcols[i] = i;
-						}
-						
-						for (col = 0; col < rnb_cols; col++) {
-							if (col)
-								g_string_append_c (retstring, sep);
-							g_string_append_c (retstring, quote);
-							g_string_append (retstring,
-									 gda_data_model_get_column_name (model, rcols[col]));
-							g_string_append_c (retstring, quote);
-						}
-						g_string_append_c (retstring, '\n');
-						if (!cols)
-							g_free (rcols);
+					if (cols) {
+						rcols = (gint *)cols;
+						rnb_cols = nb_cols;
 					}
+					else {
+						gint i;
+							
+						rnb_cols = gda_data_model_get_n_columns (model);
+						rcols = g_new (gint, rnb_cols);
+						for (i = 0; i < rnb_cols; i++)
+							rcols[i] = i;
+					}
+						
+					for (col = 0; col < rnb_cols; col++) {
+						if (col)
+							g_string_append_c (retstring, sep);
+						g_string_append_c (retstring, quote);
+						g_string_append (retstring,
+								 gda_data_model_get_column_name (model, rcols[col]));
+						g_string_append_c (retstring, quote);
+					}
+					g_string_append_c (retstring, '\n');
+					if (!cols)
+						g_free (rcols);
 				}
-				else
-					g_warning (_("The '%s' parameter must hold a boolean value, ignored."),
-						   "FIELDS_NAME");
 			}
+			else
+				g_warning (_("The '%s' parameter must hold a boolean value, ignored."),
+					   "FIELDS_NAME");
 		}
 		
 		if (cols) {
@@ -1467,73 +1458,81 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 
 		/* analyse options */
 		GdaHolder *holder;
-		holder = gda_set_get_holder (options, "ROW_NUMBERS");
+		holder = options ? gda_set_get_holder (options, "ROW_NUMBERS") : NULL;
 		if (holder) {
 			const GValue *value;
 			value = gda_holder_get_value (holder);
 			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
 				dump_rows = g_value_get_boolean ((GValue *) value);
-			else 
+			else
 				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "ROW_NUMBERS");
 		}
 
-		holder = gda_set_get_holder (options, "NAME");
+		holder = options ? gda_set_get_holder (options, "NAME") : NULL;
 		if (holder) {
 			const GValue *value;
 			value = gda_holder_get_value (holder);
 			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
 				dump_title = g_value_get_boolean ((GValue *) value);
-			else 
+			else
 				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "NAME");
 		}
 
-		holder = gda_set_get_holder (options, "NAMES_ON_FIRST_LINE");
+		holder = options ? gda_set_get_holder (options, "NAMES_ON_FIRST_LINE") : NULL;
 		if (holder) {
 			const GValue *value;
 			value = gda_holder_get_value (holder);
 			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
 				dump_column_titles = g_value_get_boolean ((GValue *) value);
-			else 
+			else
 				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "NAMES_ON_FIRST_LINE");
 		}
 
-		holder = gda_set_get_holder (options, "NULL_AS_EMPTY");
+		holder = options ? gda_set_get_holder (options, "NULL_AS_EMPTY") : NULL;
 		if (holder) {
 			const GValue *value;
 			value = gda_holder_get_value (holder);
 			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
 				null_as_empty = g_value_get_boolean ((GValue *) value);
-			else 
+			else
 				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "NULL_AS_EMPTY");
 		}
 		
-		holder = gda_set_get_holder (options, "MAX_WIDTH");
+		holder = options ? gda_set_get_holder (options, "MAX_WIDTH") : NULL;
 		if (holder) {
 			const GValue *value;
 			value = gda_holder_get_value (holder);
-			if (value && (G_VALUE_TYPE (value) == G_TYPE_INT))
+			if (value && (G_VALUE_TYPE (value) == G_TYPE_INT)) {
 				max_width = g_value_get_int ((GValue *) value);
-			else 
+#ifdef TIOCGWINSZ
+				if (max_width < 0) {
+					struct winsize window_size;
+					if (ioctl (0,TIOCGWINSZ, &window_size) == 0)
+						max_width = (int) window_size.ws_col;
+				}
+#endif
+			}
+			else
 				g_warning (_("The '%s' parameter must hold an integer value, ignored."), "MAX_WIDHT");
 		}
 
-		holder = gda_set_get_holder (options, "COLUMN_SEPARATORS");
+		holder = options ? gda_set_get_holder (options, "COLUMN_SEPARATORS") : NULL;
 		if (holder) {
 			const GValue *value;
 			value = gda_holder_get_value (holder);
 			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
 				dump_separators = g_value_get_boolean ((GValue *) value);
-			else 
+			else
 				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "SEPARATORS");
 		}
 
-		holder = gda_set_get_holder (options, "SEPARATOR_LINE");
+		holder = options ? gda_set_get_holder (options, "SEPARATOR_LINE") : NULL;
 		if (holder) {
 			const GValue *value;
 			value = gda_holder_get_value (holder);
 			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
 				dump_title_line = g_value_get_boolean ((GValue *) value);
-			else 
+			else
 				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "SEPARATOR_LINE");
 		}
 
@@ -1556,7 +1555,7 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 	}
 
 	default:
-		g_assert_not_reached ();
+		g_warning (_("Unknown GdaDataModelIOFormat %d value"), format);
 	}
 	return NULL;
 }
@@ -1573,7 +1572,10 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
  * @options: list of options for the export
  * @error: a place to store errors, or %NULL
  *
- * Exports data contained in @model to the @file file; the format is specified using the @format argument.
+ * Exports data contained in @model to the @file file; the format is specified using the @format argument. Note that
+ * the date format used is the one used by the connection from which the data model has been made (as the result of a
+ * SELECT statement), or, for other kinds of data models, the default format (refer to gda_data_handler_get_default()) unless
+ * the "cnc" property has been set and points to a #GdaConnection to use that connection's date format.
  *
  * Specifically, the parameters in the @options list can be:
  * <itemizedlist>
@@ -1623,18 +1625,16 @@ gda_data_model_export_to_file (GdaDataModel *model, GdaDataModelIOFormat format,
 	g_return_val_if_fail (file, FALSE);
 
 	body = gda_data_model_export_to_string (model, format, cols, nb_cols, rows, nb_rows, options);
-	if (options) {
-		GdaHolder *holder;
+	GdaHolder *holder;
 		
-		holder = gda_set_get_holder (options, "OVERWRITE");
-		if (holder) {
-			const GValue *value;
-			value = gda_holder_get_value (holder);
-			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
-				overwrite = g_value_get_boolean ((GValue *) value);
-			else
-				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "OVERWRITE");
-		}
+	holder = options ? gda_set_get_holder (options, "OVERWRITE") : NULL;
+	if (holder) {
+		const GValue *value;
+		value = gda_holder_get_value (holder);
+		if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
+			overwrite = g_value_get_boolean ((GValue *) value);
+		else
+			g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "OVERWRITE");
 	}
 
 	if (g_file_test (file, G_FILE_TEST_EXISTS)) {
@@ -2023,6 +2023,8 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 		if (g_ptr_array_index (values, i))
 			gda_value_free ((GValue *) g_ptr_array_index (values, i));
 
+	g_ptr_array_free (values, TRUE);
+
 	return retval;
 }
 
@@ -2731,8 +2733,19 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 								str = (gchar*) g_value_get_string (value);
 							else {
 								if (use_data_handlers) {
-									GdaDataHandler *dh;
-									dh = gda_data_handler_get_default (G_VALUE_TYPE (value));
+									GdaDataHandler *dh = NULL;
+									GdaConnection *cnc;
+									GdaServerProvider *prov;
+									cnc = g_object_get_data (G_OBJECT (model), "cnc");
+									if (!cnc && GDA_IS_DATA_SELECT (model))
+										cnc = gda_data_select_get_connection (GDA_DATA_SELECT (model));
+									if (cnc) {
+										prov = gda_connection_get_provider (cnc);
+										dh = gda_server_provider_get_data_handler_g_type (prov, cnc,
+																  G_VALUE_TYPE (value));
+									}
+									if (!dh)
+										dh = gda_data_handler_get_default (G_VALUE_TYPE (value));
 									if (dh)
 										str = gda_data_handler_get_str_from_value (dh, value);
 									else
@@ -2866,8 +2879,19 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 								str = (gchar*) g_value_get_string (value);
 							else {
 								if (use_data_handlers) {
-									GdaDataHandler *dh;
-									dh = gda_data_handler_get_default (G_VALUE_TYPE (value));
+									GdaDataHandler *dh = NULL;
+									GdaConnection *cnc;
+									GdaServerProvider *prov;
+									cnc = g_object_get_data (G_OBJECT (model), "cnc");
+									if (!cnc && GDA_IS_DATA_SELECT (model))
+										cnc = gda_data_select_get_connection (GDA_DATA_SELECT (model));
+									if (cnc) {
+										prov = gda_connection_get_provider (cnc);
+										dh = gda_server_provider_get_data_handler_g_type (prov, cnc,
+																  G_VALUE_TYPE (value));
+									}
+									if (!dh)
+										dh = gda_data_handler_get_default (G_VALUE_TYPE (value));
 									if (dh)
 										str = gda_data_handler_get_str_from_value (dh, value);
 									else
