@@ -61,7 +61,7 @@ gda_handler_numerical_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+		static GMutex registering;
 		static const GTypeInfo info = {
 			sizeof (GdaHandlerNumericalClass),
 			(GBaseInitFunc) NULL,
@@ -81,12 +81,12 @@ gda_handler_numerical_get_type (void)
 			NULL
 		};
 
-		g_static_mutex_lock (&registering);
+		g_mutex_lock (&registering);
 		if (type == 0) {
 			type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerNumerical", &info, 0);
 			g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
 		}
-		g_static_mutex_unlock (&registering);
+		g_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -186,9 +186,7 @@ gda_handler_numerical_get_sql_from_value (G_GNUC_UNUSED GdaDataHandler *iface, c
 	g_assert (value);
 
 	gchar *retval;
-	setlocale (LC_NUMERIC, "C");
 	retval = gda_value_stringify ((GValue *) value);
-	setlocale (LC_NUMERIC, gda_numeric_locale);
 
 	if (!retval)
 		retval = g_strdup ("0");
@@ -299,11 +297,7 @@ gda_handler_numerical_get_value_from_str (G_GNUC_UNUSED GdaDataHandler *iface, c
 	else if (type == G_TYPE_CHAR) {
 		if (!*endptr && (llint >= G_MININT8) && (llint <= G_MAXINT8)) {
 			value = g_value_init (g_new0 (GValue, 1), G_TYPE_CHAR);
-#if GLIB_CHECK_VERSION(2,31,7)
 			g_value_set_schar (value, (gchar) llint);
-#else
-			g_value_set_char (value, (gint8) llint);
-#endif
 		}
 	}
 	else if (type == G_TYPE_UINT64) {
