@@ -32,6 +32,7 @@
 #include "gda-firebird-recordset.h"
 #include "gda-firebird-provider.h"
 #include <libgda/libgda-global-variables.h>
+#include <libgda/gda-debug-macros.h>
 
 #define _GDA_PSTMT(x) ((GdaPStmt*)(x))
 
@@ -136,7 +137,7 @@ gda_firebird_recordset_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+		static GMutex registering;
 		static const GTypeInfo info = {
 			sizeof (GdaFirebirdRecordsetClass),
 			(GBaseInitFunc) NULL,
@@ -149,7 +150,7 @@ gda_firebird_recordset_get_type (void)
 			(GInstanceInitFunc) gda_firebird_recordset_init,
 			0
 		};
-		g_static_mutex_lock (&registering);
+		g_mutex_lock (&registering);
 		if (type == 0) {
 #ifdef FIREBIRD_EMBED
 			type = g_type_register_static (GDA_TYPE_DATA_SELECT, "GdaFirebirdRecordsetEmbed", &info, 0);
@@ -157,7 +158,7 @@ gda_firebird_recordset_get_type (void)
 			type = g_type_register_static (GDA_TYPE_DATA_SELECT, "GdaFirebirdRecordset", &info, 0);
 #endif
 		}
-		g_static_mutex_unlock (&registering);
+		g_mutex_unlock (&registering);
 	}
 
 	return type;
