@@ -7,7 +7,7 @@
  * Copyright (C) 2003 Chris Silles <csilles@src.gnome.org>
  * Copyright (C) 2003 Laurent Sansonetti <lrz@gnome.org>
  * Copyright (C) 2003 Paisa Seeluangsawat <paisa@users.sf.net>
- * Copyright (C) 2004 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2004 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2005 Alan Knowles <alan@akbkhome.com>
  * Copyright (C) 2005 - 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2005 Mike Fisk <mfisk@woozle.org>
@@ -289,7 +289,7 @@ gda_mysql_recordset_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+		static GMutex registering;
 		static const GTypeInfo info = {
 			sizeof (GdaMysqlRecordsetClass),
 			(GBaseInitFunc) NULL,
@@ -302,10 +302,10 @@ gda_mysql_recordset_get_type (void)
 			(GInstanceInitFunc) gda_mysql_recordset_init,
 			NULL
 		};
-		g_static_mutex_lock (&registering);
+		g_mutex_lock (&registering);
 		if (type == 0)
 			type = g_type_register_static (GDA_TYPE_DATA_SELECT, "GdaMysqlRecordset", &info, 0);
-		g_static_mutex_unlock (&registering);
+		g_mutex_unlock (&registering);
 	}
 
 	return type;
@@ -864,6 +864,7 @@ new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, G_GNUC_UNUSED gint rownum, G
 					.minute = bvalue.minute,
 					.second = bvalue.second,
 					.fraction = bvalue.second_part,
+					.timezone = 0 /* GMT */
 				};
 				gda_value_set_time (value, &time);
 			}
@@ -883,6 +884,7 @@ new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, G_GNUC_UNUSED gint rownum, G
 					.minute = bvalue.minute,
 					.second = bvalue.second,
 					.fraction = bvalue.second_part,
+					.timezone = 0 /* GMT */
 				};
 				gda_value_set_timestamp (value, &timestamp);
 			}
